package com.ydd.gateway.service;

import com.alibaba.fastjson.JSON;
import com.ydd.gateway.filter.AuthorizeGatewayFilterFactory;
import com.ydd.gateway.filter.ResponseObject;

import com.ydd.gateway.util.Md5Util;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.TreeMap;

@Component
public class FilterService {

	private static final Log logger = LogFactory.getLog(AuthorizeGatewayFilterFactory.class);

	private static final String AUTHORIZE_TOKEN = "clientId";

//	@Autowired
//	RedisTemplate redisTemplate;
	public Mono<DataBuffer> validateSign(TreeMap<String, String> params, String signStr, ServerHttpResponse response) {
		if (StringUtils.isEmpty(signStr)) {
			ResponseObject data = new ResponseObject();
			data.setResult("error");
			data.setMsg("非法请求");
			byte[] datas = JSON.toJSONString(data).getBytes(StandardCharsets.UTF_8);
			DataBuffer buffer = response.bufferFactory().wrap(datas);
			response.setStatusCode(HttpStatus.BAD_REQUEST);
			return Mono.just(buffer);
		}

		// logger.info("签名参数" + params.toString());
		// logger.info("签名值" + signStr);
		// logger.info("签名后的值" + Md5Util.sign(params));

		// 查看传过来的日期与当前日期比较相差几分钟
		long min = getMin(params.get("time"));
		if (min >= 3) {// 暂时定相差三分钟为无效请求
			logger.info("相差三分钟为无效请求");
			ResponseObject data = new ResponseObject();
			data.setResult("error");
			data.setMsg("非法请求");
			byte[] datas = JSON.toJSONString(data).getBytes(StandardCharsets.UTF_8);
			DataBuffer buffer = response.bufferFactory().wrap(datas);
			response.setStatusCode(HttpStatus.BAD_REQUEST);
			return Mono.just(buffer);
		}

		boolean right = Md5Util.verifySign(params, signStr);
		if (!right) {
			ResponseObject data = new ResponseObject();
			data.setResult("error");
			data.setMsg("签名不合法");
			byte[] datas = JSON.toJSONString(data).getBytes(StandardCharsets.UTF_8);
			DataBuffer buffer = response.bufferFactory().wrap(datas);
			response.setStatusCode(HttpStatus.UNAUTHORIZED);

			return Mono.just(buffer);
		}
		return null;
	}



	/**
	 * 比较与当前时间相差几分钟
	 * 
	 * @param time
	 * @return
	 * @throws Exception
	 */
	public long getMin(String time) {
		long nd = 1000 * 24 * 60 * 60;

		long nh = 1000 * 60 * 60;

		long nm = 1000 * 60;

		// long ns = 1000;

		// 获得两个时间的毫秒时间差异
		// String time = "20190321141020";
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
		long diff = 0;
		try {
			diff = cal.getTimeInMillis() - sdf.parse(time).getTime();
		}
		catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// 计算差多少天
		long day = diff / nd;

		// 计算差多少小时
		long hour = diff % nd / nh;

		// 计算差多少分钟
		long min = diff % nd % nh / nm;

		// 计算差多少秒//输出结果
		// long sec = diff % nd % nh % nm / ns;
		return min;
	}
}
